#!/usr/bin/perl

sub sublex {
    $a=~m/([0-9]+) ([0-9]+)/;
    $aa=$1;
    $ab=$2;
    $b=~m/([0-9]+) ([0-9]+)/;
    $ba=$1;
    $bb=$2;
    if($aa==$ba) {
	return $ab <=> $bb;
    } else {
	return $aa <=> $ba;
    }
}

$seed=shift;
if(!defined $seed) {
    die "no random seed given as input";
}
srand(int($seed));

$have_params=0;
$have_labels=0;
while(<STDIN>) {
    chomp;
    if(m/^p edge ([0-9]+) ([0-9]+)/) {
	if($have_params) {
	    die "duplicate DIMACS format line";
	}
	$have_params=1;
	$nv=$1;
	$ne=$2;
	$ec=0;
	for($i=0;$i<$nv;$i++) {
	    $rp[$i]=$i;
	}
	for($i=0;$i<$nv;$i++) {
	    $j=$i+int(rand($nv-$i));
	    $t=$rp[$i];
	    $rp[$i]=$rp[$j];
	    $rp[$j]=$t;
	}
    } elsif(m/^n ([0-9]+) (.+)/) {
	if(!$have_params) {
	    die "DIMACS format line missing";
	}
	$have_labels=1;
	$v=$1-1;
	$lab=$2;
	if($v>=$nv) {
	    die "invalid vertex label specification";
	}
	if(defined $vlab{$rp[$v]}) {
	    die "duplicate vertex label specification";
	}
	$vlab[$rp[$v]]=$lab;
    } elsif(m/^e ([0-9]+) ([0-9]+)/) {
	if(!$have_params) {
	    die "DIMACS format line missing";
	}
	$i=$rp[$1-1];
	$j=$rp[$2-1];
	if($i > $j) {
	    $t = $i;
	    $i = $j;
	    $j = $t;
	}
	if($i >= $nv || $j >= $nv || $i==$j) {
	    die "invalid edge specification";
	}
	if(defined $edge{"$i $j"}) {
	    die "repeated edge found";
	}
	$edge{"$i $j"}=1;
	$elist[$ec++]=($i+1)." ".($j+1);
    } elsif(m/^c /) {
	# Comment line. Ignore.
    } elsif(m/^ *$/) {
	# Blank line. Ignore.
    } else {
	die "parse error";
    }
}
if($ec!=$ne) {
    die "invalid number of edges";
}
if($have_labels) {
    for($i=0;$i<$nv;$i++) {
	if(!defined($vlab[$i])) {
	    die "missing vertex label(s)";
	}
    }
}
print "p edge $nv $ne\n";
if($have_labels) {
    for($i=0;$i<$nv;$i++) {
	print "n ".($i+1)." ".$vlab[$i]."\n";
    }
}
@elist=sort sublex @elist;
for($i=0;$i<$ne;$i++) {
    print "e $elist[$i]\n"
}
